@@ -1,4 +1,3 @@
-//!
 //! This module contains the definitions for stivale2 boot protocol.
 //!
 //! This Documentation contains miminal information about the protocol and provides brief
@@ -6,35 +5,29 @@
 //!
 //! For detailed documentation, Visit the official [docs](https://github.com/stivale/stivale/blob/master/STIVALE2.md)
 
-#[repr(C, packed)]
-pub struct Stivale2Tag {
-    pub identifier: u64,
-    pub next: u64,
-}
-
-/// The kernel executable shall have a section .stivale2hdr which will contain the header that the bootloader will parse.
-/// The following header should be initalized as static and should be linked as section
-/// `.stivale2hdr`
+/// The kernel executable shall have a section .stivale2hdr which will contain the header that the
+/// bootloader will parse. The following header should be initalized as static and should be linked
+/// as section `.stivale2hdr`
 #[repr(C, packed)]
 pub struct Stivale2Header {
     pub entry_point: u64,
 
-    /// The stack pointer should be casted to `*const ()`.
-    ///
+    /// The stack pointer should be cast to *const ().
     /// # Example
     /// ```
-    /// static STACK: [u8;1024] = [0;1024];
-    /// let ptr = &STACK as *const u8 as *const ();
-    ///
+    /// static STACK : [u8;4096] = [0;4096];
+    /// let ptr = &STACK[4095] as *const u8 as *const ();
     /// ```
     pub stack: *const (),
+
     pub flags: u64,
+
+    /// Pointer to start of a linked list.
+    ///
+    /// Should be cast into `*const ()`.
     pub tags: *const (),
 }
 
-unsafe impl Sync for Stivale2Header {}
-unsafe impl Send for Stivale2Header {}
-
 impl Stivale2Header {
     /// Create a stivale2 header structure which should be linked to the ELF Section
     /// ".stivale2hdr".
@@ -46,7 +39,7 @@ impl Stivale2Header {
     ///
     ///#[used]
     ///#[link_section = ".stivale2hdr"]
-    ///static hdr : Stivale2Header = Stivale2Header::new(0,&stack, 1 << 0 , core::mem::null());
+    ///static hdr : Stivale2Header = Stivale2Header::new(core::ptr::null(),&stack, 1 << 0 , core::mem::null());
     ///
     /// ```
     pub const fn new<const SIZE: usize>(
@@ -57,17 +50,29 @@ impl Stivale2Header {
     ) -> Self {
         Stivale2Header {
             entry_point,
+            stack: &stack[4095] as *const u8 as *const (),
             flags,
             tags,
-            stack: stack as *const u8 as *const (),
         }
     }
 }
 
+unsafe impl Sync for Stivale2Header {}
+unsafe impl Send for Stivale2Header {}
+
+#[repr(C, packed)]
+pub struct Stivale2Tag {
+    pub identifier: u64,
+    pub next: *const (),
+}
+
+unsafe impl Sync for Stivale2Tag {}
+unsafe impl Send for Stivale2Tag {}
+
 pub const STIVALE2_HEADER_TAG_FRAMEBUFFER_ID: u64 = 0x3ecc1bc43d0f7971;
 
 #[repr(C, packed)]
-pub struct Stivale2HeaderTagFramebuffer {
+pub struct Stivale2HeaderTagFrameBuffer {
     pub identifier: u64,
     pub next: *const (),
     pub framebuffer_width: u16,
@@ -75,10 +80,8 @@ pub struct Stivale2HeaderTagFramebuffer {
     pub framebuffer_bpp: u16,
 }
 
-unsafe impl Sync for Stivale2HeaderTagFramebuffer {}
-unsafe impl Send for Stivale2HeaderTagFramebuffer {}
-
-pub const STIVALE2_HEADER_TAG_FB_MTRR_ID: u64 = 0x4c7bb07731282e00;
+unsafe impl Sync for Stivale2HeaderTagFrameBuffer {}
+unsafe impl Send for Stivale2HeaderTagFrameBuffer {}
 
 pub const STIVALE2_HEADER_TAG_TERMINAL_ID: u64 = 0xa85d499b1823be72;
 
@@ -92,32 +95,10 @@ pub struct Stivale2HeaderTagTerminal {
 unsafe impl Sync for Stivale2HeaderTagTerminal {}
 unsafe impl Send for Stivale2HeaderTagTerminal {}
 
-pub const STIVALE2_HEADER_TAG_SMP_ID: u64 = 0x1ab015085f3273df;
-
-#[repr(C, packed)]
-pub struct Stivale2HeaderTagSmp {
-    pub identifier: u64,
-    pub next: *const (),
-    pub flags: u64,
-}
-
-unsafe impl Sync for Stivale2HeaderTagSmp {}
-unsafe impl Send for Stivale2HeaderTagSmp {}
-
-pub const STIVALE2_HEADER_TAG_5LV_PAGING_ID: u64 = 0x932f477032007e8f;
-
-pub const STIVALE2_HEADER_TAG_UNMAP_NULL_ID: u64 = 0x92919432b16fe7e7;
-
-pub const STIVALE2_BOOTLOADER_BRAND_SIZE: usize = 64;
-pub const STIVALE2_BOOTLOADER_VERSION_SIZE: usize = 64;
-
-/// The info passed on by the bootloader.
-///
-/// `tags` should be iterated to get the desired tags.
 #[repr(C, packed)]
 pub struct Stivale2Struct {
-    pub bootloader_brand: [u8; STIVALE2_BOOTLOADER_BRAND_SIZE],
-    pub bootloader_version: [u8; STIVALE2_BOOTLOADER_VERSION_SIZE],
+    pub bootloader_brand: [u8; 64],
+    pub bootloader_version: [u8; 64],
     pub tags: u64,
 }
 
@@ -138,84 +119,47 @@ impl Stivale2Struct {
     ///}
     /// ```
     pub fn get_tag(&self, id: u64) -> *const () {
-        let mut current = id as *const ();
+        let mut current = self.tags as *const ();
 
         loop {
+            // If the linked list is null or the id is not found in the list.
             if current.is_null() {
                 return core::ptr::null();
             }
 
             let _c = current as *const Stivale2Tag;
+
             unsafe {
                 if (*_c).identifier == id {
                     return current;
                 }
-
                 current = (*_c).next as *const ();
             }
         }
     }
 }
 
-pub const STIVALE2_STRUCT_TAG_CMDLINE_ID: u64 = 0xe5e76a1b4597a781;
-
-#[repr(C, packed)]
-pub struct Stivale2StructTagCmdline {
-    pub identifier: u64,
-    pub next: *const (),
-    pub cmdline: u64,
-}
-
-unsafe impl Sync for Stivale2StructTagCmdline {}
-unsafe impl Send for Stivale2StructTagCmdline {}
-
-pub const STIVALE2_STRUCT_TAG_MEMMAP_ID: u64 = 0x2187f79e8612de07;
-
-pub const STIVALE2_MMAP_USABLE: u64 = 1;
-pub const STIVALE2_MMAP_RESERVED: u64 = 2;
-pub const STIVALE2_MMAP_ACPI_RECLAIMABLE: u64 = 3;
-pub const STIVALE2_MMAP_ACPI_NVS: u64 = 4;
-pub const STIVALE2_MMAP_BAD_MEMORY: u64 = 5;
-pub const STIVALE2_MMAP_BOOTLOADER_RECLAIMABLE: u64 = 0x100;
-pub const STIVALE2_MMAP_KERNEL_AND_MODULES: u64 = 0x100;
-pub const STIVALE2_MMAP_FRAMEBUFFER: u64 = 0x100;
-
 pub const STIVALE2_STRUCT_TAG_TERMINAL_ID: u64 = 0xc2b3f4c3233b0974;
 
 #[repr(C, packed)]
 pub struct Stivale2StructTagTerminal {
     pub identifier: u64,
-    pub next: *const (),
+    pub next: u64,
     pub flags: u32,
     pub cols: u16,
     pub rows: u16,
     pub term_write: u64,
 }
 
-unsafe impl Sync for Stivale2StructTagTerminal {}
-unsafe impl Send for Stivale2StructTagTerminal {}
-
 impl Stivale2StructTagTerminal {
-    /// Get a `terminal_write` function from the function pointer provided by the bootloader.
-    ///
-    /// Returns a closure that can be used to print to the terminal by passing a `&str`.
-    ///
-    ///# Example
-    ///
-    /// ```
-    /// let print = terminal.get_term_write();
-    /// print("Hello world");
-    ///
-    /// ```
-    pub fn get_term_write(&self) -> impl Fn(&str) {
-        type Stivale2TermWrite = extern "C" fn(*const i8, u64);
+    pub fn get_term_func(&self) -> impl Fn(&str) {
+        let _t = self.term_write as *const ();
 
-        let _func = unsafe {
-            core::mem::transmute::<*const (), Stivale2TermWrite>(self.term_write as *const ())
-        };
+        let _term_func =
+            unsafe { core::mem::transmute::<*const (), extern "C" fn(*const i8, u64)>(_t) };
 
-        move |msg| {
-            _func(msg.as_ptr() as *const i8, msg.len() as u64);
+        move |txt| {
+            _term_func(txt.as_ptr() as *const i8, txt.len() as u64);
         }
     }
 }
